from typing import List, Optional
from sqlalchemy.orm import Session

from app.models.todo import Todo
from app.schemas.todo import TodoCreate, TodoUpdate


class TodoRepository:
    def get_todos(self, db: Session, skip: int = 0, limit: int = 100) -> List[Todo]:
        return db.query(Todo).offset(skip).limit(limit).all()
    
    def get_todo_count(self, db: Session) -> int:
        return db.query(Todo).count()
    
    def get_todo(self, db: Session, todo_id: int) -> Optional[Todo]:
        return db.query(Todo).filter(Todo.id == todo_id).first()
    
    def create_todo(self, db: Session, todo_in: TodoCreate) -> Todo:
        db_todo = Todo(
            title=todo_in.title,
            description=todo_in.description,
            completed=todo_in.completed
        )
        db.add(db_todo)
        db.commit()
        db.refresh(db_todo)
        return db_todo
    
    def update_todo(
        self, db: Session, *, db_obj: Todo, obj_in: TodoUpdate
    ) -> Todo:
        update_data = obj_in.model_dump(exclude_unset=True)
        for field in update_data:
            setattr(db_obj, field, update_data[field])
        db.add(db_obj)
        db.commit()
        db.refresh(db_obj)
        return db_obj
    
    def delete_todo(self, db: Session, *, todo_id: int) -> None:
        todo = db.query(Todo).filter(Todo.id == todo_id).first()
        if todo:
            db.delete(todo)
            db.commit()


todo_repository = TodoRepository()